/****************************************************************************
**
** Copyright (C) 2015 The Qt Company Ltd.
** Contact: http://www.qt.io/licensing
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://www.qt.io/licensing.  For further information
** use the contact form at http://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 or version 3 as published by the Free
** Software Foundation and appearing in the file LICENSE.LGPLv21 and
** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
** following information to ensure the GNU Lesser General Public License
** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
****************************************************************************/

#include <sqlitecolumn.h>

#include <gmock/gmock.h>
#include <gmock/gmock-matchers.h>
#include <gtest/gtest.h>
#include "gtest-qt-printing.h"

namespace {

class SqliteColumn : public ::testing::Test
{
protected:
    void SetUp() override;

    ::SqliteColumn column;
};

TEST_F(SqliteColumn, ChangeName)
{
    column.setName(Utf8StringLiteral("Claudia"));

    ASSERT_THAT(column.name(), Utf8StringLiteral("Claudia"));
}

TEST_F(SqliteColumn, DefaultType)
{
    ASSERT_THAT(column.type(), ColumnType::Numeric);
}

TEST_F(SqliteColumn, ChangeType)
{
    column.setType(ColumnType::Text);

    ASSERT_THAT(column.type(), ColumnType::Text);
}

TEST_F(SqliteColumn, DefaultPrimaryKey)
{
    ASSERT_FALSE(column.isPrimaryKey());
}

TEST_F(SqliteColumn, SetPrimaryKey)
{
    column.setIsPrimaryKey(true);

    ASSERT_TRUE(column.isPrimaryKey());
}

TEST_F(SqliteColumn, GetColumnDefinition)
{
    column.setName(Utf8StringLiteral("Claudia"));

    Internal::ColumnDefinition columnDefintion = column.columnDefintion();

    ASSERT_THAT(columnDefintion.name(), Utf8StringLiteral("Claudia"));
    ASSERT_THAT(columnDefintion.type(), ColumnType::Numeric);
    ASSERT_FALSE(columnDefintion.isPrimaryKey());
}

void SqliteColumn::SetUp()
{
    column.clear();
}

}
