#ifndef COMPLETE_TARGET_HEADER_CHANGED_H
#define COMPLETE_TARGET_HEADER_CHANGED_H

void FunctionInIncludedHeader();
void FunctionInIncludedHeaderChanged();

#endif // COMPLETE_TARGET_HEADER_CHANGED_H

