/**************************************************************************
**
** Copyright (C) 2012 - 2014 BlackBerry Limited. All rights reserved.
**
** Contact: BlackBerry (qt@blackberry.com)
** Contact: KDAB (info@kdab.com)
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company.  For licensing terms and
** conditions see http://www.qt.io/terms-conditions.  For further information
** use the contact form at http://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 or version 3 as published by the Free
** Software Foundation and appearing in the file LICENSE.LGPLv21 and
** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
** following information to ensure the GNU Lesser General Public License
** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, The Qt Company gives you certain additional
** rights.  These rights are described in The Qt Company LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
****************************************************************************/

#ifndef QNX_INTERNAL_QNXQTVERSION_H
#define QNX_INTERNAL_QNXQTVERSION_H

#include "qnxconstants.h"
#include "qnxqtversion.h"

#include <qtsupport/baseqtversion.h>

#include <utils/environment.h>

namespace Qnx {
namespace Internal {

class QnxQtVersion : public QtSupport::BaseQtVersion
{
public:
    QnxQtVersion();
    QnxQtVersion(QnxArchitecture arch, const Utils::FileName &path,
                 bool isAutoDetected = false,
                 const QString &autoDetectionSource = QString());
    QnxQtVersion *clone() const override;

    QString type() const override;

    QString description() const override;

    Core::FeatureSet availableFeatures() const override;
    QString platformName() const override;
    QString platformDisplayName() const override;

    QString qnxHost() const;
    QString qnxTarget() const;

    QnxArchitecture architecture() const;
    QString archString() const;

    QVariantMap toMap() const override;
    void fromMap(const QVariantMap &map) override;

    QList<ProjectExplorer::Abi> detectQtAbis() const override;

    void addToEnvironment(const ProjectExplorer::Kit *k, Utils::Environment &env) const override;
    Utils::Environment qmakeRunEnvironment() const override;

    QtSupport::QtConfigWidget *createConfigurationWidget() const override;

    bool isValid() const;
    QString invalidReason() const;

    QString sdkPath() const;
    void setSdkPath(const QString &sdkPath);

private:
    void updateEnvironment() const;

    QList<Utils::EnvironmentItem> environment() const;

    QnxArchitecture m_arch;
    QString m_sdkPath;

    mutable bool m_environmentUpToDate;
    mutable QList<Utils::EnvironmentItem> m_qnxEnv;
};

} // namespace Internal
} // namespace Qnx

#endif // QNX_INTERNAL_QNXQTVERSION_H
