#ifdef PROJECT_CONFIGURATION_1
int projectConfiguration1;
#elif defined(PROJECT_CONFIGURATION_2)
int projectConfiguration2;
#else
int noProjectConfigurationDetected;
#endif

void f()
{
    /* COMPLETE HERE */
}
