#ifndef NODEINSTANCEGLOBAL_H
#define NODEINSTANCEGLOBAL_H

#include <QByteArray>
#include <QList>

namespace QmlDesigner {

typedef QByteArray PropertyName;
typedef QList<PropertyName> PropertyNameList;
typedef QByteArray TypeName;

}

#endif // NODEINSTANCEGLOBAL_H
