IDI_ICON1               ICON    DISCARDABLE     "qtcreator.ico"
IDI_ICON2               ICON    DISCARDABLE     "winicons/c.ico"
IDI_ICON3               ICON    DISCARDABLE     "winicons/cpp.ico"
IDI_ICON4               ICON    DISCARDABLE     "winicons/h.ico"
IDI_ICON5               ICON    DISCARDABLE     "winicons/ui.ico"
IDI_ICON6               ICON    DISCARDABLE     "winicons/pro.ico"
IDI_ICON7               ICON    DISCARDABLE     "winicons/pri.ico"
IDI_ICON8               ICON    DISCARDABLE     "winicons/qml.ico"
