/****************************************************************************
**
** Copyright (C) 2015 The Qt Company Ltd.
** Contact: http://www.qt.io/licensing
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company.  For licensing terms and
** conditions see http://www.qt.io/terms-conditions.  For further information
** use the contact form at http://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 or version 3 as published by the Free
** Software Foundation and appearing in the file LICENSE.LGPLv21 and
** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
** following information to ensure the GNU Lesser General Public License
** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, The Qt Company gives you certain additional
** rights.  These rights are described in The Qt Company LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
****************************************************************************/

#ifndef PROJECTEXPLORER_KITCHOOSER_H
#define PROJECTEXPLORER_KITCHOOSER_H

#include "projectexplorer_export.h"

#include <QWidget>

#include <functional>

QT_BEGIN_NAMESPACE
class QComboBox;
class QPushButton;
QT_END_NAMESPACE

namespace Core { class Id; }

namespace ProjectExplorer {

class Kit;

// Let the user pick a kit.
class PROJECTEXPLORER_EXPORT KitChooser : public QWidget
{
    Q_OBJECT

public:
    explicit KitChooser(QWidget *parent = 0);

    void setCurrentKitId(Core::Id id);
    Core::Id currentKitId() const;

    typedef std::function<bool(const Kit *k)> KitMatcher;
    void setKitMatcher(const KitMatcher &matcher);

    Kit *currentKit() const;

signals:
    void currentIndexChanged(int);
    void activated(int);

public slots:
    void populate();

private slots:
    void onCurrentIndexChanged(int index);
    void onManageButtonClicked();

protected:
    virtual QString kitText(const Kit *k) const;
    virtual QString kitToolTip(Kit *k) const;

private:
    KitMatcher m_kitMatcher;
    Kit *kitAt(int index) const;
    QComboBox *m_chooser;
    QPushButton *m_manageButton;
};

} // namespace ProjectExplorer

#endif // PROJECTEXPLORER_KITCHOOSER_H
