#ifndef COMPLETE_TARGET_HEADER_UNSAVED_H
#define COMPLETE_TARGET_HEADER_UNSAVED_H

void FunctionInIncludedHeader();
void FunctionInIncludedHeaderUnsaved();

#endif // COMPLETE_TARGET_HEADER_UNSAVED_H

