body{
font: normal 400 14px/1.2 Arial;
margin-top:85px;
font-family: Arial, Helvetica;
color:#313131;
text-align:justify;
margin-left:5px;
margin-right:5px;
}

img{
-moz-box-shadow: 3px 3px 3px #ccc;
-webkit-box-shadow: 3px 3px 3px #ccc;
box-shadow: 3px 3px 3px #ccc;
border:#8E8D8D 2px solid;
margin-left:0px;
max-width: 800px;
height: auto
}

b{
font-weight:600;
}

.content{}

.descr{
margin-top:35px;
/*max-width: 75%;*/
margin-left:5px;
text-align:justify;
min-height:700px;
vertical-align:top;
}

.name{
max-width: 75%;
font-weight:100;
}

tt{
text-align:left;}

/*
-----------
links
-----------
*/

a:link{
color: #2C418D;
text-decoration: none;
text-align:left;
}

a:hover{
color: #869CD1;
text-decoration:underline;
text-align:left;
}

a:visited{
color: #869CD1;
text-decoration: none;
text-align:left;
}

a:visited:hover{
text-decoration:underline;
text-align:left;
}

a[href*="http://"], a[href*="ftp://"],a[href*="https://"]
{
text-decoration: none;
background-image:url(../images/ico_out.png);
background-repeat:no-repeat;
background-position:left;
padding-left:20px;
text-align:left;
}

/*a[href*="http://qt.nokia.com/doc/"], a[href*="http://doc.qt.nokia.com/"]
{
background: none;
padding-left: 0px;
text-align:left;
}*/

.flags{
text-decoration:none;
text-height:24px;
}

/*
-------------------------------
NOTE styles
-------------------------------
*/
.notetitle, .tiptitle, .fastpathtitle{
font-weight:bold;
}

.attentiontitle,.cautiontitle,.dangertitle,.importanttitle,.remembertitle,.restrictiontitle{
font-weight:bold;
}

.note,.tip,.fastpath{
background: #F2F2F2 url(../images/ico_note.png);
background-repeat: no-repeat;
background-position: top left;
padding:5px;
padding-left:40px;
padding-bottom:10px;
border:#999 1px dotted;
color:#666666;
margin:5px;
}

.attention,.caution,.danger,.important,.remember,.restriction{
background: #F2F2F2 url(../images/ico_note_attention.png);
background-repeat:no-repeat;
background-position:top left;
padding:5px;
padding-left:40px;
padding-bottom:10px;
border:#999 1px dotted;
color:#666666;
margin:5px;
}

/*
-------------------------------
Top navigation
-------------------------------
*/

.header{

height:1px;
padding:0px;
margin:0px;
}

.qtref{
display: block;
position: relative;
top: -76px;
height:15px;
z-index: 1;
font-size:11px;
padding-right:10px;
float:right;
}

.naviNextPrevious{
display: block;
position: relative;
text-align: right;
top: -53px;
float:right;
height:20px;
z-index:1;
padding-right:10px;
padding-top:2px;
vertical-align:top;
margin:0px;
}


.naviNextPrevious > a:first-child{
background-image:url(../images/btn_prev.png);
background-repeat:no-repeat;
background-position:left;
padding-left:20px;
height:20px;
padding-left:20px;
}

.naviNextPrevious > a:last-child{
background-image:url(../images/btn_next.png);
background-repeat:no-repeat;
background-position:right;
padding-right:20px;
height:20px;
margin-left:30px;
}

.breadcrumb{
display: block;
position: relative;
top:-20px;
/*border-top:2px solid #ffffff;*/
border-bottom: 1px solid #cecece;
background-color:#F2F2F2;
z-index:1;
height:20px;
padding:0px;
margin:0px;
padding-left:10px;
padding-top:2px;
margin-left:-5px;
margin-right:-5px;
}

.breadcrumb ul{
 margin:0px;
 padding:0px;
}

.breadcrumb ul li{
background-color:#F2F2F2;
list-style-type:none;
padding:0;
margin:0;
height:20px;
}

.breadcrumb li{
float:left;
}

.breadcrumb .first {
background:url(../images/home.png);
background-position:left;
background-repeat:no-repeat;
padding-left:20px;
}


.breadcrumb li a{
color:#2C418D;
display:block;
text-decoration:none;
background:url(../images/arrow.png);
background-repeat:no-repeat;
background-position:right;
padding-right:25px;
padding-left:10px;
}

.breadcrumb li a:hover{
color:#909090;
display:block;
text-decoration:none;
background:url(../images/arrow.png);
background-repeat:no-repeat;
background-position:right;
padding-right:20px;
padding-left:10px;
}


/* table of content
no display
*/

/*
-----------
headers
-----------
*/


@media screen{
.title{
color:#313131;
font-size: 18px;
font-weight: normal;
left: 0;
padding-bottom: 20px;
padding-left: 10px;
padding-top: 20px;
position: absolute;
right: 0;
top: 0;
background-color:#E6E6E6;
border-bottom: 1px #CCC solid;
border-top: 2px #CCC solid;
font-weight:bold;
margin-left:0px;
margin-right:0px;
}
}

h1 {
margin: 0;
}

h2, p.h2 {
font: 500 16px/1.2 Arial;
font-weight:100;
background-color:#F2F3F4;
padding:4px;
margin-bottom:30px;
margin-top:30px;
border-top:#E0E0DE 1px solid;
border-bottom: #E0E0DE 1px solid;
max-width: 99%;
}

h3{

font: 500 14px/1.2 Arial;
font-weight:100;
text-decoration:underline;
margin-bottom:30px;
margin-top:30px;
}

h3.fn,span.fn{
border-width: 1px;
border-style: solid;
border-color: #E6E6E6;
-moz-border-radius: 7px 7px 7px 7px;
-webkit-border-radius: 7px 7px 7px 7px;
border-radius: 7px 7px 7px 7px;
background-color: #F6F6F6;
word-spacing: 3px;
padding: 5px 5px;
text-decoration:none;
font-weight:bold;
max-width:75%;
font-size:14px;
margin:0px;
margin-top:30px;

}

.name{
color:#1A1A1A;
}
.type{
color:#808080;
}



@media print {
.title {
color:#0066CB;
font-family:Arial, Helvetica;
font-size: 32px;
font-weight: normal;
left: 0;
position: absolute;
right: 0;
top: 0;
}
}


/*
-----------------
table styles
-----------------
*/
.table img {
border:none;
margin-left:0px;
-moz-box-shadow:0px 0px 0px #fff;
-webkit-box-shadow: 0px 0px 0px #fff;
box-shadow: 0px 0px 0px #fff;
}

/* table with border alternative colours*/

 table,pre{
-moz-border-radius: 7px 7px 7px 7px;
-webkit-border-radius: 7px 7px 7px 7px;
 border-radius: 7px 7px 7px 7px;
background-color: #F6F6F6;
border: 1px solid #E6E6E6;
border-collapse: separate;
font-size: 12px;
line-height: 1.2;
margin-bottom: 25px;
margin-left: 15px;
font-size: 12px;
line-height: 1.2;
margin-bottom: 25px;
margin-left: 15px;
}


table th{
text-align:left;
padding-left:20px;
}

table td {
padding: 3px 15px 3px 20px;
border-bottom:#CCC dotted 1px;
}
table p { margin:0px;}

table tr.even {
background-color: white;
color: #66666E;
}

table tr.odd {
background-color: #F6F6F6;
color: #66666E;
}


table thead {
text-align:left;
padding-left:20px;
background-color:#e1e0e0;
border-left:none;
border-right:none;
}

table thead th {
padding-top:5px;
padding-left:10px;
padding-bottom:5px;
border-bottom: 2px solid #D1D1D1;
padding-right:10px;
}


/* table bodless & white*/

.borderless {
border-radius: 0px 0px 0px 0px;
background-color: #fff;
border: 1px solid #fff;
}

.borderless tr {
background-color: #FFF;
color: #66666E;
}

.borderless td {
border:none;
border-bottom:#fff dotted 1px;
}

/*
-----------
List
-----------
*/

ul{
padding-bottom:2px;
}

li ul {
padding-top: 10px;
}
li {
margin-bottom: 10px;
padding-left: 8px;
list-style:outside;
list-style-type:square;
text-align:left;
}


ol{
margin:10px;
padding:0;
}

ol > li{
margin-left: 30px;
padding-left:8px;
list-style:decimal;
}

.centerAlign{
text-align: left;
}

.cpp{
display: block;
margin: 10;
overflow: hidden;
overflow-x: hidden;
overflow-y: hidden;
padding: 20px 0 20px 0;
}

.footer{
margin-top: 50px;
padding-left:5px;
margin-bottom: 10px;
font-size:10px;
border-top: 1px solid #999;
padding-top:11px;
}

.footerNavi{
width:auto;
text-align:right;
margin-top:50px;
z-index:1;
}

.memItemLeft{
padding-right: 3px;
}

.memItemRight{
padding: 3px 15px 3px 0;
}

.qml{
display: block;
margin: 10;
overflow: hidden;
overflow-x: hidden;
overflow-y: hidden;
padding: 20px 0 20px 0;
}

.qmldefault{
padding-left: 5px;
float: right;
color: red;
}

.qmlreadonly{
padding-left: 5px;
float: right;
color: #254117;
}

.rightAlign{
padding: 3px 5px 3px 10px;
text-align: right;
}

/*
-----------
Content table
-----------
*/

@media print{
.toc {
float: right;
padding-bottom: 10px;
padding-top: 50px;
width: 100%;
background-image:url(../images/bgrContent.png);
background-position:top;
background-repeat:no-repeat;
}
}

@media screen{
.toc{
clear:both;
float:right;
vertical-align:top;
-moz-border-radius: 7px 7px 7px 7px;
-webkit-border-radius: 7px 7px 7px 7px;
border-radius: 7px 7px 7px 7px;
 background:#FFF url(../images/bgrContent.png);
background-position:top;
background-repeat:repeat-x;
border: 1px solid #E6E6E6;
padding-left:5px;
padding-bottom:10px;
height: auto;
width: 200px;
text-align:left;
z-index:2;
margin-left:20px;
margin-right:20px;
margin-top:0px;
padding-top:0px;
}
}

.toc h3{
text-decoration:none;
}

.toc h3{font: 500 14px/1.2 Arial;
font-weight:100;
padding:0px;
margin:0px;
padding-top:5px;
padding-left:5px;
}


.toc ul{
width:160px;
padding-left:10px;
padding-right:5px;
padding-bottom:10px;
padding-top:10px;
}

.toc ul li{
margin-left:20px;
list-style-image:url(../images/blu_dot.png);
list-style:outside;

}


.toc ul li a:link{
color: #2C418D;
text-decoration: none;
}

.toc ul li a:hover{
color: #869CD1;
text-decoration:underline;

}

.toc ul li a:visited{
color: #869CD1;
font-weight: bold;
}

.level1{
border:none;}

.clearfix{
clear:both;}

