QWidget {
    background-color: #363636;
    border: none;
}

QTabWidget::pane {
    background-color: #363636;
    border: none;
}

QAbstractScrollArea {
    background-color: #363636;
    border: none;
}

QPlainTextEdit {
    background-color: #ababab;
    border: none;
}

QTabBar::tab {
    border-image: none;

    border-top: 1px solid #000000;
    border-left: 1px solid #000000;
    border-right: 1px solid #000000;

    min-width: 80px;

    height: 22px;
    margin-top: 2px;
    margin-bottom: -1px;
    padding-left: 2px;
    padding-right: 4px;

    font: bold;
    font-size: 11px;
    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
    stop: 0 #3c3c3c, stop: 1 #444444);

    color: #ababab;
}

QTabBar::tab:hover {
    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
    stop: 0 #3c3c3c, stop: 1 #666666);
}

QTabBar::tab:selected {
    border: none;
    background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
    stop: 0 #EEEEEE, stop: 1 #ababab);
    color: #3c3c3c;
}

