/****************************************************************************
**
** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://www.qt.io/licensing.  For further information
** use the contact form at http://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 or version 3 as published by the Free
** Software Foundation and appearing in the file LICENSE.LGPLv21 and
** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
** following information to ensure the GNU Lesser General Public License
** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
****************************************************************************/

#ifndef TIMELINERENDERPASS_H
#define TIMELINERENDERPASS_H

#include "timeline_global.h"
#include <QVector>

QT_FORWARD_DECLARE_CLASS(QSGNode)
namespace Timeline {

class TimelineAbstractRenderer;
class TimelineRenderState;

class TIMELINE_EXPORT TimelineRenderPass {
public:
    class TIMELINE_EXPORT State {
    public:
        virtual const QVector<QSGNode *> &expandedRows() const;
        virtual const QVector<QSGNode *> &collapsedRows() const;
        virtual QSGNode *expandedOverlay() const;
        virtual QSGNode *collapsedOverlay() const;
        virtual ~State();
    };

    virtual ~TimelineRenderPass();
    virtual State *update(const TimelineAbstractRenderer *renderer,
                          const TimelineRenderState *parentState,
                          State *state, int indexFrom, int indexTo, bool stateChanged,
                          qreal spacing) const = 0;
};

} // namespace Timeline

#endif // TIMELINERENDERPASS_H
