/****************************************************************************
**
** Copyright (C) 2015 The Qt Company Ltd.
** Contact: http://www.qt.io/licensing
**
** This file is part of Qt Creator.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company.  For licensing terms and
** conditions see http://www.qt.io/terms-conditions.  For further information
** use the contact form at http://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 or version 3 as published by the Free
** Software Foundation and appearing in the file LICENSE.LGPLv21 and
** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
** following information to ensure the GNU Lesser General Public License
** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, The Qt Company gives you certain additional
** rights.  These rights are described in The Qt Company LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
****************************************************************************/

#include "osparser.h"
#include "projectexplorerconstants.h"
#include "task.h"

#include <utils/hostosinfo.h>

using namespace ProjectExplorer;

OsParser::OsParser() :
    m_hasFatalError(false)
{
    setObjectName(QLatin1String("OsParser"));
}

void OsParser::stdError(const QString &line)
{
    if (Utils::HostOsInfo::isLinuxHost()) {
        const QString trimmed = line.trimmed();
        if (trimmed.contains(QLatin1String(": error while loading shared libraries:"))) {
            addTask(Task(Task::Error, trimmed, Utils::FileName(), -1, Constants::TASK_CATEGORY_COMPILE));
        }
    }
    IOutputParser::stdError(line);
}

void OsParser::stdOutput(const QString &line)
{
    if (Utils::HostOsInfo::isWindowsHost()) {
        const QString trimmed = line.trimmed();
        if (trimmed == QLatin1String("The process cannot access the file because it is being used by another process.")) {
            addTask(Task(Task::Error, tr("The process can not access the file because it is being used by another process.\n"
                                         "Please close all running instances of your application before starting a build."),
                         Utils::FileName(), -1, Constants::TASK_CATEGORY_COMPILE));
            m_hasFatalError = true;
        }
    }
    IOutputParser::stdOutput(line);
}

bool OsParser::hasFatalErrors() const
{
    return m_hasFatalError || IOutputParser::hasFatalErrors();
}
